precision mediump float;

uniform sampler2D      sTexture;
uniform mediump vec2   MinTexCoord;
uniform mediump vec2   MaxTexCoord;
uniform mediump float  TimePortion;
uniform bool           ReverseEffect;

uniform bool           UseColor;
uniform mediump vec3   Color;

varying mediump vec2   TexCoord;

void main()
{
    float scale = 6.0;
    float y = scale * (MaxTexCoord.y - TexCoord.y) / (MaxTexCoord.y - MinTexCoord.y); // y is in [0; 6]
    y = fract(y); // get only factional part of the y - y is in [0; 1]
        
    // check reverse alpha
    float reverseEffect = float(ReverseEffect); // reverse alpha is 0 or 1
    float t = reverseEffect * (1.0 - TimePortion) + (1.0 - reverseEffect) * TimePortion; // t is TimePortion or (1 - TimePortion)
    
    // process as wipe
    t = (1.0 + scale) * t; // t is in [0; 7]
    y = scale * y; // y is in [0; 6]
    
    float alpha = clamp(t - y, 0.0, 1.0);
    
    ////////////////////////////
    // Apply color
    vec4 textureColor = texture2D(sTexture, TexCoord);
    
    float useColor = float(UseColor); // useColor is 0 or 1
    
    // The following code line says that:
    // color = textureColor, if UseColor is false
    // color = vec4(Color * textureColor.a, textureColor.a), if UseColor is true
    vec4 color = (1.0 - useColor) * textureColor + useColor * vec4(Color * textureColor.a, textureColor.a);
    ////////////////////////////
    
    gl_FragColor = color * alpha;
}
