class __MS_Color
{

	
	constructor() {
		this.transparent = [ "T" ];
		this.black = [ "G", 0 ];
		this.white = [ "G", 1 ];
		this.red = [ "RGB", 1, 0, 0 ];
		this.green = [ "RGB", 0, 1, 0 ];
		this.blue = [ "RGB", 0, 0, 1 ];
		this.cyan = [ "CMYK", 1, 0, 0, 0 ];
		this.magenta = [ "CMYK", 0, 1, 0, 0 ];
		this.yellow = [ "CMYK", 0, 0, 1, 0 ];
		this.dkGray = [ "G", 0.25 ];
		this.gray = [ "G", 0.5 ];
		this.ltGray = [ "G", 0.75 ];
	}
    
	convert(colorArray, cColorspace)
	{
	
		if(!__MS_Color.checkColorArrayValidity(colorArray) || !__MS_Color.checkColorSpaceValidity(cColorspace))
			return [];
	
	
		var orgColorSpace = colorArray[0];
		
		if(orgColorSpace == cColorspace)
			return colorArray;
	
		
	
		if(orgColorSpace == "CMYK")
		{
			var c = colorArray[1];
			var m = colorArray[2];
			var y = colorArray[3];
			var k = colorArray[4];
			if(cColorspace == "RGB")
			{
				
				var r = 1.0 - Math.min(1.0, c + k);
				var g = 1.0 - Math.min(1.0, m + k);
				var b = 1.0 - Math.min(1.0, y + k);
				return ["RGB", r, g, b];
			}
			else if(cColorspace == "G")
			{
				var gray = 1.0 - Math.min(1.0, 0.3*c + 0.59*m + 0.11*y + k);
				return ["G", gray];
			}
		}
	
	
		else if(orgColorSpace == "RGB")
		{
			var r = colorArray[1];
			var g = colorArray[2];
			var b = colorArray[3];
			if(cColorspace == "CMYK")
			{
				var c = 1.0 - r;
				var m = 1.0 - g;
				var y = 1.0 - b;
				var k = 0; // Not calculated according to JavaScript API documentation
				return ["CMYK", c, m, y, k];
			}
			else if(cColorspace == "G")
			{
				var gray = 0.3*r + 0.59*g + 0.11*b;
				return ["G", gray];
			}
		}
		else if(orgColorSpace == "G")
		{
			var gray = colorArray[1];
			if(cColorspace == "CMYK")
			{
				var c = 0;
				var m = 0;
				var y = 0;
				var k = 1.0 - gray;
				return ["CMYK", c, m, y, k];
			}
			else if(cColorspace == "RGB")
			{
				var r = gray;
				var g = gray;
				var b = gray;
				return ["RGB", r, g, b];
			}
		}
	}
	
	
	equal(colorArray1, colorArray2)
	{
		if((colorArray1.length == 1) && (colorArray2.length == 1) && (colorArray1[0] == "T") && (colorArray2[0] == "T"))
			return true; // Both are transparent
	
		if(!__MS_Color.checkColorArrayValidity(colorArray1) || !__MS_Color.checkColorArrayValidity(colorArray2))
			return false;
	
		if(!__MS_Color.checkColorSpaceValidity(colorArray1[0]) || !__MS_Color.checkColorSpaceValidity(colorArray2[0]))
			return false;
		
		
		
		var sameColorSpaceEqual = function(colorArray1, colorArray2)
		{
			if(colorArray1.length != colorArray2.length)
				return false;
			
			var areEqual = true;
			for(var i = 0; i < colorArray1.length; i++)
			{
				if(colorArray1[i] != colorArray2[i])
				{
					areEqual = false;
					break;
				}
			}
			return areEqual;
		}
		
		
		if(colorArray1.length == colorArray2.length)
		{
			return sameColorSpaceEqual(colorArray1, colorArray2);
		}
		else
		{
			if(colorArray1.length > colorArray2.length)
			{
				var convertedColorArray = convert(colorArray1, colorArray2[0]);
				return sameColorSpaceEqual(colorArray2, convertedColorArray);
			}
			else
			{
				var convertedColorArray = convert(colorArray2, colorArray1[0]);
				return sameColorSpaceEqual(colorArray1, convertedColorArray);
			}
		}
		return false;
	}
	
	static checkColorArrayValidity(colorArray)
	{
		if((colorArray == null) || (colorArray == undefined))
			return false;
		if(!Array.isArray(colorArray))
			return false;
		if(colorArray.length == 0)
			return false;
		if((colorArray[0] != "CMYK") && (colorArray[0] != "RGB") && (colorArray[0] != "G"))
			return false;
		if((colorArray[0] == "CMYK") && (colorArray.length != 5))
			return false;
		if((colorArray[0] == "RGB") && (colorArray.length != 4))
			return false;
		if((colorArray[0] == "G") && (colorArray.length != 2))
			return false;
		
		var isValid = true;
		for(var i = 1; i < colorArray.length; i++)
		{
			if(typeof(colorArray[i]) != "number")
			{
				isValid = false;
				break;
			}
		}
		
		return isValid;
	}

	
	
	static checkColorSpaceValidity(cColorspace)
	{
		if((cColorspace == "CMYK") || (cColorspace == "RGB") || (cColorspace == "G"))
			return true;

		return false;
	}	
}

const color = new __MS_Color();
this.color = color;
Object.freeze(color);

